#ifndef ENEMY_H
#define ENEMY_H

#include "include.h"
#include "geometry.h"
#include "loading.h"
#include "particle.h"
#include "math.h"

enum{ DEAD=0, SPAWN, ORIENT, STRIKE };

enum{ _PLAYER=0, _BASIC, _BIRD, _SNAKE, _JUMPER };

struct PLAYER;

struct ENEMY{
    QUAD num;
    VECTOR dp;
    double eye;
    double eyeTo;
    int last;

    bool hit;

    PLAYER* player;

    PCONTROLLER* pc;

    unsigned char mode;
    unsigned char lives;
    unsigned char kind;

    int iter;
    int process;

    bool grounded;

    static double ddp;

    void spawn(double x, PCONTROLLER* _pc, PLAYER* _player, int m=1);

    void tick();
    void render();
    void renderNum();
    void renderEye();
};

struct JUMPER:ENEMY{

    void spawn(double x, PCONTROLLER* _pc, PLAYER* _player, int m=1);
    void tick();
    void render();
    void renderEye();
};

struct BASIC:ENEMY{

};

enum{ SWIPE=0, STAB, SWING };

struct SWORDBASE{
    VECTOR s[3];
    VECTOR b[3];

    double side;

    SWORDBASE();
    VECTOR getSVector(int iter, int process, unsigned char type, int facing);
    VECTOR getBVector(int iter, int process, unsigned char type, int facing);
};

struct PLAYER:ENEMY{
    double foot1;
    double foot2;

    QUAD f1;
    QUAD f2;

    bool liftingLeft;

    VECTOR sword;
    VECTOR base;
    SWORDBASE sb;
    unsigned char pause;
    unsigned char swingType;

    int facing;

    void spawn(double x, PCONTROLLER* _pc);

    void swing(bool left);
    void jump();
    void fall();
    void move(int d);
    void struckHere(VECTOR v);

    void tick();
    void render();
    void renderEye();
};

struct BIRD:ENEMY{
    VECTOR wing;

    void spawn(double x, PCONTROLLER* _pc, PLAYER* _player, int m=1);
    void tick();
    void render();
    void renderEye();
};

struct SNAKE:ENEMY{
    std::vector<VECTOR> pos;
    std::vector<VECTOR> def;

    VECTOR defhead;
    VECTOR tohead;
    VECTOR head;
    VECTOR headPoint;

    int facing;
    int h;
    int w;

    void spawn(double x, PCONTROLLER* _pc, PLAYER* _player);
    void tick();
    void render();
    void renderEye();
};

#endif // ENEMY_H
