#include "loading.h"

SDL_Cursor* getCursorFromBitmap(char* fname){
    int i = -1;
    Uint8 data[4*32];
    Uint8 mask[4*32];//data[i] |= 0x01;
    int hot_x = 0, hot_y = 0;

    printf("%s ", fname);
    FILE* f = fopen( fname, "rb" );
    if( f <= 0 ){ printf("failed\n"); return NULL; }
    printf("loaded\n");
    unsigned char head [26];
    fread( head, 26, 1, f );
    if( head[0] != 0x42 || head[1] != 0x4D ) return NULL;
    int wid = head[18] + ( head[19] << 8 ) + ( head[20] << 16 ) + ( head[21] << 24 );
    int hgt = head[22] + ( head[23] << 8 ) + ( head[24] << 16 ) + ( head[25] << 24 );
    int off = head[10] + ( head[11] << 8 ) + ( head[12] << 16 ) + ( head[13] << 24 );
    fseek( f, off, SEEK_SET );

    for (int b = 0; b < hgt; b++){
    //for (int b = hgt-1; b >= 0; b--){
        for (int a = 0; a < wid; a++){
        //for (int a = wid-1; a >= 0; a--){
            if (a%8){
                data[i] <<= 1;
                mask[i] <<= 1;
            }
            else{
                i++;
                data[i] = mask[i] = 0;
            }
            fgetc( f );
            if ((unsigned char)fgetc( f ) == 192){
                hot_x = a;
                hot_y = b;
            }
            switch((unsigned char)fgetc( f )){
                case 0:
                    data[i] |= 0x01;
                    mask[i] |= 0x01;
                    break;
                case 127:
                    break;
                case 255:
                    mask[i] |= 0x01;
                    break;
            }

        }
        fseek( f, (4 - ((wid * 3) % 4))%4, SEEK_CUR );
    }

    fclose( f );

    return SDL_CreateCursor(data, mask, wid, hgt, hot_x, hot_y);
}

Uint32 getpixel(SDL_Surface *surface, int x, int y){
    int bpp = surface->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to retrieve */
    Uint8 *p = (Uint8 *)surface->pixels + y*surface->pitch + x*bpp;

    switch(bpp) {
    case 1:
        return *p;
        break;

    case 2:
        return *(Uint16 *)p;
        break;

    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
            return p[0] << 16 | p[1] << 8 | p[2];
        else
            return p[0] | p[1] << 8 | p[2] << 16;
        break;

    case 4:
        return *(Uint32 *)p;
        break;

    default:
        return 0;       /* shouldn't happen, but avoids warnings */
    }
}
void putpixel(SDL_Surface *surface, int x, int y, Uint32 pixel){
    int bpp = surface->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to set */
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

    switch(bpp) {
    case 1:
        *p = pixel;
        break;

    case 2:
        *(Uint16 *)p = pixel;
        break;

    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
            p[0] = (pixel >> 16) & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = pixel & 0xff;
        } else {
            p[0] = pixel & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = (pixel >> 16) & 0xff;
        }
        break;

    case 4:
        *(Uint32 *)p = pixel;
        break;
    }
}
SDL_Surface* scaleSurface(SDL_Surface* surface, int scale){
    SDL_Surface *returnThis = SDL_CreateRGBSurface(surface->flags, surface->w*scale, surface->h*scale, surface->format->BitsPerPixel, surface->format->Rmask, surface->format->Gmask, surface->format->Bmask, surface->format->Amask);

    //SDL_CreateRGBSurface()

    for (int i = 0; i < surface->w; i++){
        for (int j = 0; j < surface->h; j++){

            Uint32 color = getpixel(surface,i,j);

            for (int k = 0; k < scale; k++){
                for (int l = 0; l < scale; l++){
                    putpixel(returnThis, scale*i + k, scale*j + l,color);
                }
            }

        }
    }

    return returnThis;
}

GLuint glLoadImageFile(char* fname, unsigned int mode, GLuint defaul){

    GLenum texture_format;
    GLint  nOfColors;
    GLuint texture;

    ///printf("      Loading %s as an SDL_Surface...\n", fname);

    SDL_Surface* surface = SDL_LoadBMP( fname );
    if (!surface){ return defaul; }

    Uint32 rmask, gmask, bmask, amask;

    #if SDL_BYTEORDER == SDL_BIG_ENDIAN
        rmask = 0xff000000;
        gmask = 0x00ff0000;
        bmask = 0x0000ff00;
        amask = 0x000000ff;
    #else
        rmask = 0x000000ff;
        gmask = 0x0000ff00;
        bmask = 0x00ff0000;
        amask = 0xff000000;
    #endif

    SDL_Surface* formatted;

    //Transparentcy
    if (mode&0x01){

        formatted = SDL_CreateRGBSurface( surface->flags, surface->w, surface->h, 32, rmask, gmask, bmask, amask );

        for (int y = 0; y < surface->h; y++){
            for (int x = 0; x < surface->w; x++){
                //Uint32 pix = ((unsigned int*)surface->pixels)[y*(surface->pitch/sizeof(unsigned int)) + x];
                //color = translate_color(getpixel(surface,x,y));
                Uint32 pixel = getpixel(surface,x,y);
                Uint8 r; Uint8 g; Uint8 b;

                SDL_GetRGB(pixel, surface->format, &r, &g, &b);

                if (r == 255 && g == 0 && b == 255){
                    //printf("Magenta");
                    putpixel(formatted,x,y,0x0000000000&(~amask));
                }
                else{
                    putpixel(formatted,x,y,pixel|amask);
                }
                    //((unsigned int*)surface->pixels)[y*(surface->pitch/sizeof(unsigned int)) + x] = SDL_MapRGBA(surface->format, 0, 0, 0, 0);
            }
        }
    }
    else{
        formatted = surface;
    }

//    if (mode&0x08){
//        formatted = scaleSurface(formatted,16);
//    }
//    else if (mode&0x04){
//        formatted = scaleSurface(formatted,8);
//    }
//    else if (mode&0x02){
//        formatted = scaleSurface(formatted,4);
//    }

	formatted = scaleSurface(formatted,8);

    if (surface && formatted){

        if ( (formatted->w & (formatted->w - 1)) != 0 ) { printf("        warning: image.bmp's width is not a power of 2\n"); }
        if ( (formatted->h & (formatted->h - 1)) != 0 ) { printf("        warning: image.bmp's height is not a power of 2\n"); }

        nOfColors = formatted->format->BytesPerPixel;
        if (nOfColors == 4){      texture_format = 0x80E1; }
        else if (nOfColors == 3){ texture_format = 0x80E0; }
        else { printf("       warning: the image is not truecolor..  this will probably break\n"); }

        //printf("With %i colors!", nOfColors);

        //texture_format = 0x1908;

        glGenTextures(1, &texture);
        glBindTexture(GL_TEXTURE_2D, texture);
        if (mode&0x10){
            glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
            glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
        }
        else{
            glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
            glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
        }

        glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST/*or GL_LINEAR*/);
        glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR/*or GL_NEAREST*/);
        //glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR/*or GL_NEAREST*/);
        //glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR/*or GL_NEAREST*/);

        glTexParameterf(GL_TEXTURE_2D, 0x84FE, 4);

        //gluBuild2DMipmaps( GL_TEXTURE_2D, 0, formatted->w, formatted->h, GL_RGB, GL_UNSIGNED_BYTE, formatted->pixels );
        //formatted.

        glTexImage2D(GL_TEXTURE_2D, 0, nOfColors, formatted->w, formatted->h, 0, texture_format, GL_UNSIGNED_BYTE, formatted->pixels);

        SDL_FreeSurface(formatted);
        SDL_FreeSurface(surface);

        ///printf("      OpenGL texture created from file.\n");

        glBindTexture(GL_TEXTURE_2D, NULL);
    }
    else{
         printf("      failed.\n");
    }
    //printf("\n");
    if (texture){ return texture; }
    return defaul;
}

void rect(double rx, double ry, double w, double h, GLubyte c1, GLubyte c2, GLubyte c3, bool filled, GLubyte alpha){
    glBegin( filled?GL_QUADS:GL_LINE_LOOP );
    glColor4ub(c1,c2,c3,alpha);
    glVertex2d(rx, ry);
    glVertex2d(rx, ry + h);
    glVertex2d(rx + w, ry + h);
    glVertex2d(rx + w, ry);
    glEnd();
}
void justRect(double rx, double ry, double w, double h, bool filled){
    glBegin( filled?GL_QUADS:GL_LINE_LOOP );
    glVertex2d(rx, ry);
    glVertex2d(rx, ry + h);
    glVertex2d(rx + w, ry + h);
    glVertex2d(rx + w, ry);
    glEnd();
}

void FONT::set(char* fname, int ffwid, int ffhgt){

    fwid = ffwid;
    fhgt = ffhgt;
    //frhgt = height/fhgt;

    FILE* f = fopen( fname, "rb" );
    unsigned char head [26];
    fread( head, 26, 1, f );
    int wid = head[18] + ( head[19] << 8 ) + ( head[20] << 16 ) + ( head[21] << 24 );
    int hgt = head[22] + ( head[23] << 8 ) + ( head[24] << 16 ) + ( head[25] << 24 );
    int off = head[10] + ( head[11] << 8 ) + ( head[12] << 16 ) + ( head[13] << 24 );

    int c1; int c2; int c3;
    int x1; int y1; int x2; int y2;

    for (int a = 0; a < 128; a++){
        fseek( f, off, SEEK_SET );
        asciilist[a] = glGenLists(1);
        glNewList(asciilist[a], GL_COMPILE);

        x1 = (fwid)*(a%16);
        y1 = (fhgt)*(a/16);
        x2 = (fwid)*(a%16 + 1)-1;
        y2 = (fhgt)*(a/16 + 1)-1;

        length[a] = 0;

        for (int c = hgt-1; c >= 0; c--){
            for (int b = 0; b < wid; b++){

                c1 = (unsigned char)fgetc( f );
                c2 = (unsigned char)fgetc( f );
                c3 = (unsigned char)fgetc( f );

                if (b >= x1 && b <= x2 && c >= y1 && c <= y2){
                    if (c1 == 0 && c2 == 0 && c3 == 0){
                        justRect(1*(b-x1),1*(c-y1),1,1,true);
                    }
                    else if (!(c1 == 255 && c2 == 255 && c3 == 255) && length[a] < b - x1 + 1){
                        length[a] = b - x1 + 1;
                    }
                }
            }
            fseek( f, (4 - ((wid * 3) % 4))%4, SEEK_CUR );
        }
        glEndList();
    }
    fclose( f );
}
int FONT::renderWithColor(char *string, float x, float y, int c1, int c2, int c3, int alpha){
    int i = 0;
    int ascii = (int)string[i];

    int Y = 0; int X = 0;

    //glColor4ub(c1,c2,c3,alpha);

    while (ascii){

        glMatrixMode(GL_MODELVIEW);
        glPushMatrix();
        glTranslatef(X+x, Y+y, -.1);
        if (ascii == 10){
            Y += fhgt;
            X = 0;
        }
        else if (ascii >= 0 && ascii < 128){
            glCallList(asciilist[ascii]);
            X += length[ascii];
            //printf("%i", length[ascii]);
        }
        glPopMatrix();

        i++;
        ascii = (int)string[i];
    }
    return X;
}
int FONT::getLength(char *string){
    int i = 0;
    int ascii = (int)string[i];

    int X = 0;

    while (ascii){

        if (ascii == 10){
            X = 0;
        }
        else if (ascii >= 0 && ascii < 128){
            X += length[ascii];
        }

        i++;
        ascii = (int)string[i];
    }
    return X;
}
int FONT::getHeight(char *string){
    int i = 0;
    int ascii = (int)string[i];

    int Y = 0;

    while (ascii){

        if (ascii == 10){
            Y += fhgt;
        }

        i++;
        ascii = (int)string[i];
    }
    return Y;
}

void RANDTEXTURE::zero(){
	for (int i = 0; i < 16; i++){
		prob[i] = 0;
		texture[i] = NULL;
	}
}
int RANDTEXTURE::loadFromLocation(char* subFolder, int l[], GLuint defaul){
	zero();

	int total = 0; char str[32];

	for (int i = 0; i < 16; i++){
		total += (l[i]);
//		printf("%i ", l[i]);
	}

	//printf("total: %i", total);

	int i = 0;

	do{
		prob[i] = ((double)l[i])/((double)total);
		snprintf(str,32,"%s%i.bmp",subFolder,i);
		printf("    Loading texture from '%s' with probability %.1f%%.\n", str, 100*prob[i]);
		snprintf(str,32,"%s%i.bmp",subFolder,i); texture[i] = glLoadImageFile(str,0x04,defaul);
		i++;
	}while (l[i] != 0);
}

int ANIMATION::loadFromLocation(char* folder, GLuint defaul, int _num){
	num = _num;
	char str[32];
	for (int i = 0; i < num; i++){
		snprintf(str,32,"%s%i.bmp", folder, i);
		//printf("fish:%s",str);
		texture[i] = glLoadImageFile(str, 0x09, defaul);
		//system("pause");
	}

	return 1;
}

