#include "particle.h"

PCONTROLLER::PCONTROLLER(double _waterLvl){
	waterLvl = _waterLvl;
	for (int i = 0; i < 4; i++){
        ddpos[i].set(randFloat(0,10),randFloat(0,1));
	}
}
void PCONTROLLER::addOne(unsigned char kind, VECTOR at){
    PARTICLE newParticle;
    newParticle.pos = at;
	newParticle.kind = kind;

    switch (kind){
        case WATER:
            newParticle.dpos.set(0,0);
            newParticle.rand = 0;
            //if (spec){ newParticle.iter = randInt(0,628); }
            //else{ newParticle.iter = 0; }
            newParticle.size = randInt(1,3);
            break;
        case DEATH:
            newParticle.dpos.set(0,0);
            newParticle.rand = randInt(1,4);
            newParticle.iter = randInt(0,64);
            newParticle.size = randInt(1,4);
            break;
        case WIND:
            newParticle.dpos.set(-1,-1);
            newParticle.size = randInt(1,3);
            newParticle.rand = randInt(0,3);
            break;
        case PART:
            newParticle.dpos.set(randFloat(-1,1),randFloat(0,1));
            newParticle.dpos.unitEquals();
            newParticle.dpos *= .5;
            newParticle.rand = randInt(1,4);
            newParticle.iter = randInt(0,64);
            newParticle.size = randInt(1,4);
            break;
        case SPLASH:
            newParticle.dpos.set(randFloat(-1,1),randFloat(-1,1));
            newParticle.dpos.unitEquals();
            newParticle.dpos *= .03;
            newParticle.rand = randInt(64,128);
            newParticle.iter = randInt(0,64);
            newParticle.size = randInt(1,4);
            break;
        case KEY:
            newParticle.dpos.set(randFloat(-1,1),randFloat(-1,1));
            newParticle.dpos.unitEquals();
            newParticle.dpos *= .05;
            newParticle.rand = randInt(0,64);
            newParticle.iter = randInt(0,64);
            newParticle.size = randInt(2,4);
            break;
    }

    particles.push_back(newParticle);
}
void PCONTROLLER::addAtPoint(unsigned char kind, VECTOR at, int num){
	for (int i = 0; i < num; i++){
        addOne(kind, at);
	}
}
void PCONTROLLER::addInArea(unsigned char kind, VECTOR a, VECTOR b, int num, bool spec){
	VECTOR n;

	for (int i = 0; i < num; i++){
		n.set(randFloat(a.x,b.x), randFloat(a.y,b.y));
        addOne(kind, n);
	}
}
void PCONTROLLER::addInQuad(unsigned char kind, QUAD quad, int num){
	VECTOR n;

    VECTOR full = ((quad.e - quad.b) * quad.p);
    VECTOR len = (full - (quad.e - quad.b)) / 2;
    VECTOR perp = (quad.e - quad.b).perp() / 2;

    double w2 = quad.w;

    if (!quad.h){
        w2 = quad.a / (quad.e - quad.b).magn();
    }

	for (int i = 0; i < num; i++){
		n = quad.e - len + full*randFloat(0,1) + perp*(randFloat(-1,1)*.5*w2);
        addOne(kind, n);
	}
}
void PCONTROLLER::tick(){
    iter++;

    for (int i = 0; i < 4; i++){
        ddpos[i].x += randFloat(0,.5);
        ddpos[i].y += randFloat(0,.05);
        ddpos[i].unitEquals();
        ddpos[i] *= .1;
	}

	for (int i = 0; i < particles.size(); i++){
		if (!(particles[i].kind == PART && particles[i].pos.y < waterLvl -1 && particles[i].dpos.y < 0)){
			particles[i].pos += particles[i].dpos;
		}
		bool over = false;
		switch (particles[i].kind){
            case DEATH:
                if (particles[i].iter > 64){
                    particles[i].dpos.y -= .1;
                }
				over = (particles[i].iter > 128);
				break;
            case WIND:
                particles[i].dpos -= ddpos[particles[i].rand];

                particles[i].dpos.unitEquals();
                particles[i].dpos *= 2;

				over = (particles[i].pos.x < -B || particles[i].pos.y < -B);
				break;
            case PART:
				particles[i].dpos.y -= .1;
				particles[i].dpos.unitEquals();
				particles[i].dpos *= particles[i].rand/2.;
				over = (particles[i].iter > 64);
				break;
			case WATER:
				over = (particles[i].iter > 628);
				break;
			case SPLASH:
				particles[i].dpos.y -= .002;
				particles[i].dpos.unitEquals();
				particles[i].dpos *= .03;
				over = (particles[i].iter > 64);
				break;
			case KEY:
				over = (particles[i].iter > 64);
				break;
		}
		particles[i].iter++;

		if (over){ particles.erase(particles.begin() + i); i--; }

	}
}
void PCONTROLLER::render(){
	glBegin(GL_QUADS);

	for (int i = 0; i < particles.size(); i++){
//		switch (particles[i].kind){
//			case WATER:  glColor4ub(255,255,255,-128*cos(particles[i].iter*.01)+128); break;
//			case SPLASH: glColor3ub(0 + 2*particles[i].rand, 127 + particles[i].rand, 255); break;
//			case KEY: 	 glColor3ub(255, 127 + particles[i].rand, 2*particles[i].rand); break;
//		}

        double s = particles[i].size;

		glVertex2d(particles[i].pos.x + s, particles[i].pos.y - s);
		glVertex2d(particles[i].pos.x - s, particles[i].pos.y - s);
		glVertex2d(particles[i].pos.x - s, particles[i].pos.y + s);
		glVertex2d(particles[i].pos.x + s, particles[i].pos.y + s);
	}

	glEnd();
}

